@@ -4,13 +4,15 @@ use 5.014;
 use warnings;
 use strict;
 
-our $VERSION = '0.87';
+our $VERSION = '1.00';
 
 use Carp;
 use Ref::Util qw /is_arrayref/;
 use List::Util qw /min sum/;
 use List::MoreUtils qw /first_index/;
 use Scalar::Util qw /blessed looks_like_number/;
+#use parent qw/Clone/;
+use Clone ();
 
 sub new {
     my ($class, %args) = @_;
@@ -23,7 +25,7 @@ sub new {
     croak 'data argument is an empty array'
       if !scalar @$data;
 
-    my $first_neg_idx = first_index {$_ < 0} @$data;
+    my $first_neg_idx = first_index {($_ // 0) < 0} @$data;
     croak "negative values passed in data array ($data->[$first_neg_idx] at index $first_neg_idx)\n"
       if $first_neg_idx >= 0;
     
@@ -44,6 +46,29 @@ sub new {
     return $self;
 }
 
+sub clone {
+    my $self = shift;
+    my $clone;
+    
+    if ($self->{prng}->can('clone')) {
+        $clone = do {
+            delete local $self->{prng};
+            Clone::clone ($self);
+        };
+        $clone->{prng} = $self->{prng}->clone;
+    }
+    else {
+        $clone = Clone::clone ($self);
+    }
+    return $clone;
+}
+
+sub set_prng {
+    my ($self, $prng) = @_;
+    $self->_validate_prng_object($prng);
+    $self->{prng} = $prng;
+}
+
 sub _validate_prng_object {
     my ($self, $prng) = @_;
 
@@ -95,7 +120,8 @@ sub get_class_count {
 
 #  simplified version of draw_n_samples
 #  as we need a single index result
-sub draw {
+#  superseded by faster method
+sub draw_slow {
     my ($self) = @_;
 
     my $prng = $self->{prng};
@@ -129,7 +155,7 @@ sub draw {
 #  but maintaining a sorted index for binary search
 #  leads to overheads when the list is changed.
 #  Could try a red-black tree or similar...
-sub draw1 {
+sub draw {
     my ($self) = @_;
 
     my $prng = $self->{prng};
@@ -158,16 +184,15 @@ sub draw1 {
     else {
         #  work from the end
         $rand = (1 - $rand) * $norm;
-        my $cumsum = $data->[-1];
+        my $cumsum;
         my $kk = scalar @$data;  # start one beyond the end
         foreach my $data_kk (reverse @$data) {
             $kk--;
             next if !$data_kk;
+            $cumsum += $data_kk;
 
             return $kk
-              if $rand < $cumsum;
-
-            $cumsum += $data_kk;
+              if $rand <= $cumsum;
         }
     }
 
@@ -340,7 +365,10 @@ method implemented in L<Statistics::Sampler::Multinomial::AliasMethod>,
 presumably because the calls to the PRNG are inside XS and avoid
 perl subroutine overheads
 (and profiling showed the RNG calls to be the main bottleneck
-for the Alias method).  
+for the Alias method).
+
+There is a subclass that uses a hierarchical index for the draw() method
+in L<Statistics::Sampler::Multinomial::Indexed>.
 
 For more details and background about the various approaches,
 see L<http://www.keithschwarz.com/darts-dice-coins>.
@@ -369,21 +397,32 @@ stream used, and can use it as part of a separate analysis.
 The only requirement of such an object is that it has a binomial()
 method.
 
+=item $object->clone
+
+Create a clone of the sampler object.
+
+=item $object->set_prng($prng)
+
+Set the PRNG object to be used.
+Useful to reset the PRNG after a call
+to the clone method.
+
 =item $object->draw
 
 Draw one sample from the distribution.
 Returns the sampled class number (array index).
 
-=item $object->draw1
+=item $object->draw_slow
 
 Draw one sample from the distribution.
 Returns the sampled class number (array index).
 
-Same as the draw method, but faster since it uses only
+This was the draw method prior to version 1.00.
+The current method is faster since it uses only
 one call to the random number generator (profiing shows that
 to be a key slow point).  Note that the
 random draw sequence differs between the two methods so repeatability
-is affected.  It might become the default in version 1, though. 
+is affected.
 
 
 =item $object->draw_n_samples ($n)
